var args = arguments[0] || {};
var defaultTemplate = args.defaultTemplate ? args.defaultTemplate : null;

/**
 *	This is the data to display. Format is very much like the listView data
 *	{	"properties":{"templateName":"<optional>"}
 *		"element1":{<element1properties}
 *		"element2":{<element2properties}
 *		"element3":{<element3properties}
 * 	}
 *
 */
var data = args.data ? args.data : [];

/**
 *	Function that takes (item, index, list) as its parameters returns its placement. It can return {top, left, bottom, 
 *	right, width, height, center} as placement parameters, using all the same units as normal alloy code.  This 
 *  method must return sufficient information to determine the location, width, and height of the item
 */
var placement = args.placement;


//
// view creation
//

//
// view recycling
//

//
// view placement
//



var dc = Ti.Platform.displayCaps;
var densityFactor = OS_ANDROID ? dc.logicalDensityFactor : ("high" === dc.density ? 2 : 1);


/**
 *  Converts {top, left, bottom, right, width, height, center} into a Dimension using the same rules as in a composite
 *  layout in view. Given the spec, it must be possible to calculate the width and the height or this will result in an
 *  exception. For example, if only the top, bottom, xor center.y are specified, the height cannot be calculated.
 */
function calculateBounds(spec) {

	var result = {};

	// vertical
	(function(){
		var viewHeight = $.scrollView.rect.height * densityFactor;
		
		if(undefined !== spec.height) {
			
			result.height = spec.height;			
	
			if(undefined !== spec.top) {
				result.y = spec.top;
				
			} else if(undefined !== spec.center && undefined !== spec.center.y) {
				var ph = calculatePixelSize(spec.height);
				var pcy = calculatePixelSize(spec.center.y);
				result.y = (pcy - (ph/2)).toString() + 'px';
					
			} else if(undefined !== spec.bottom) {
				var ph = calculatePixelSize(spec.height);
				var pb = viewHeight - calculatePixelSize(spec.bottom);
				result.y = (pb - ph).toString() + 'px';
				
			} else {
				// only height specified, this isn't clear in the documentation, but it appears that it should be
				// centered vertically
				var ph = calculatePixelSize(spec.height);
				result.y = ((viewHeight-ph)/2).toString() + 'px';
							
			}
			
		} else if(undefined !== spec.top) {
			
			result.y = spec.top;
			var pt = calculatePixelSize(spec.top);
			
			if(undefined !== spec.center && undefined !== spec.center.y) {
				var pcy = calculatePixelSize(spec.center.y);
				result.height = (2*(pcy - pt)).toString() + 'px';			
				
			} else if(undefined !== spec.bottom) {
				result.height = (viewHeight - calculatePixelSize(spec.bottom) - pt) + 'px';
				
			} else {
				// exception			
				throw new Error("Cannot calculcate height from given spec.1");			
			}
			
		} else if(undefined !== spec.center && undefined !== spec.center.y) {
			
			if(undefined !== spec.bottom) {
				var pcy = calculatePixelSize(spec.center.y);
				var pb = viewHeight - calculatePixelSize(spec.bottom);
				result.height = (2*(pb-pcy)).toString() + 'px';
				result.y = (2*pcy - pb).toString() + 'px';
				
			} else {
				// exception			
				throw new Error("Cannot calculcate height from given spec.2");			
			}
			
		} else {
			// exception			
			throw new Error("Cannot calculcate height from given spec.3");
						
		}
	})();
		
	// horizontal
	(function(){
		var viewWidth = $.scrollView.rect.width * densityFactor;
		
		if(undefined !== spec.width) {
			
			result.width = spec.width;			
	
			if(undefined !== spec.left) {
				result.x = spec.left;
				
			} else if(undefined !== spec.center && undefined !== spec.center.x) {
				var pw = calculatePixelSize(spec.width);
				var pcx = calculatePixelSize(spec.center.x);
				result.x = (pcx - (ph/2)).toString() + 'px';
					
			} else if(undefined !== spec.right) {
				var pw = calculatePixelSize(spec.width);
				var pr = viewWidth - calculatePixelSize(spec.right);
				result.x = (pr - pw).toString() + 'px';
				
			} else {
				// only height specified, this isn't clear in the documentation, but it appears that it should be
				// centered vertically
				var pw = calculatePixelSize(spec.width);
				result.x = ((viewWidth-pw)/2).toString() + 'px';
							
			}
			
		} else if(undefined !== spec.left) {
			
			result.x = spec.left;
			var pl = calculatePixelSize(spec.left);
			
			if(undefined !== spec.center && undefined !== spec.center.x) {
				var pcx = calculatePixelSize(spec.center.x);
				result.width = (2*(pcx - pl)).toString() + 'px';			
				
			} else if(undefined !== spec.bottom) {
				result.width = (viewWidth - calculatePixelSize(spec.right) - pl) + 'px';
				
			} else {
				// exception			
				throw new Error("Cannot calculcate width from given spec.4");			
			}
			
		} else if(undefined !== spec.center && undefined !== spec.center.x) {

			if(undefined !== spec.right) {
				var pcx = calculatePixelSize(spec.center.x);
				var pr = viewWidth - calculatePixelSize(spec.right);
				result.width = (2*(pr-pcx)).toString() + 'px';
				result.x = (2*pcx - pr).toString() + 'px';
				
			} else {
				// exception			
				throw new Error("Cannot calculcate width from given spec.5");			
			}
			
		} else {
			// exception			
			throw new Error("Cannot calculcate width from given spec.6");
						
		}
	})();
	
	return result;
}

/**
 *	Translate sizes/coordinates to pixels
 *	@param spec {string} the size/coordinate to translate
 *	@param direction {string} either "veritcal" or "horizonal"
 */
function calculatePixelSize(spec, direction) {
	
	var specUnits = 'dp';
	var specValue = 0;
	if(typeof spec === 'string') {
		var unitsMatch = /^([0-9]+\.?[0-9]*)([a-z%]+)$/.exec(spec);
		if(unitsMatch && unitsMatch.length > 0) {
			specUnits = unitsMatch[2];
			specValue = unitsMatch[1];
		} else {
			specValue = new Number(spec);
			if(isNaN(specValue)) {
				throw new Error("spec " + spec + " is not valid.");
			}
		}
	} else if(typeof spec === 'number') {
		specUnits = 'dp';
		specValue = spec;
	}

	var result = -1;
	var dpi = OS_ANDROID ? ("horizontal" === direction ? dc.xdpi : dc.ydpi) : dc.dpi;  

	switch(specUnits) {
	case 'px':
		result = specValue;
		break;
	case 'dip':
	case 'dp':
		result = specValue * densityFactor;
		break;
	case 'in': 
		result = specValue*dpi;
		break;
	case 'mm':
		result = specValue*dpi/25.4;
		break;
	case 'cm':
		result = specValue*dpi/2.54;
		break;
	case 'pt':
		result = specValue*dpi/72.0;
		break;
	case '%':
		var containerSize = densityFactor*($.scrollView.rect[("horizontal" === direction ? "width" : "height")]);
		result = specValue*containerSize/100;
		break;
	default:
		throw new Error("spec " + spec + " has unknown units");
	}

	return Math.round(result);
}

exports.calculatePixelSize = calculatePixelSize;
exports.calculateBounds = calculateBounds;
