
require("test/ti-mocha");

var should = require('test/should');

var scroller = Alloy.createController('quickview/scroller');
var window = Ti.UI.createWindow({backgroundColor:'black'});
window.open({modal:true,layout:'vertical'});
var scrollerParent = Ti.UI.createView({width: 100, height: 100,backgroundColor:'red'});
window.add(scrollerParent);
window.add(Ti.UI.createLabel({text:'running tests',color:'yellow'}));
scrollerParent.add(scroller.getView());

describe("quickview/scroller", function() {

	var dc = Ti.Platform.displayCaps;
	var densityFactor = OS_ANDROID ? dc.logicalDensityFactor : ("high" === dc.density ? 2 : 1);
	var dpi = OS_ANDROID ? dc.xdpi : dc.dpi; 


	describe("#pixelSize", function() {
		var s = function(size, expected) {
			Math.round(Math.abs(scroller.calculatePixelSize(size, 'horizontal') - expected)).toString().should.equal('0');
		};
		it('should calculate pixel sizes correctly - number', function() {s(20, 20*densityFactor);});
		it('should calculate pixel sizes correctly - string', function() {s('20', 20*densityFactor);});
		it('should calculate pixel sizes correctly - dp    ', function() {s('20dp', 20*densityFactor);});
		it('should calculate pixel sizes correctly - dip   ', function() {s('20dip', 20*densityFactor);});
		it('should calculate pixel sizes correctly - px    ', function() {s('20px', 20);});
		it('should calculate pixel sizes correctly - in    ', function() {s('20in', 20*dpi);});
		it('should calculate pixel sizes correctly - cm    ', function() {s('20cm', 20*dpi/2.54);});
		it('should calculate pixel sizes correctly - mm    ', function() {s('20mm', 20*dpi/25.4);});
		it('should calculate pixel sizes correctly - pt    ', function() {s('20pt', 20*dpi/72);});
		it('should calculate pixel sizes correctly - %     ', function() {s('20%', 20*densityFactor);});
	});
	
	
	describe("#boundingBox", function() {
		var json = require('json/json-stable-stringify');
		var s = function(bounds, rect) {
			var b = scroller.calculateBounds(bounds), r = {};
			for(var i in b) {
				b[i] = scroller.calculatePixelSize(b[i], "horizontal").toString()+'px';
			}
			for(var i in rect) {
				r[i] = scroller.calculatePixelSize(rect[i], "horizontal").toString()+'px';
			}
			json(b).should.equal(json(r));
		};
		var rect1 = {x:20,y:20,width:20,height:20};
		it('should calculate bounds correctly ht', function() {s({top:20,left:20,width:20,height:20}, rect1);});
		it('should calculate bounds correctly hc', function() {s({center:{x:30,y:30},top:20,left:20}, rect1);});
		it('should calculate bounds correctly hb', function() {s({bottom:60,right:60,width:20,height:20}, rect1);});
		it('should calculate bounds correctly h', function() {s({width:20,height:20}, {x:40,y:40,height:20,width:20});});
		it('should calculate bounds correctly tc', function() {s({top:20,left:20,center:{x:30,y:30}}, rect1);});
		it('should calculate bounds correctly tb', function() {s({top:20,left:20,bottom:60,right:60}, rect1);});
		it('should calculate bounds correctly cb', function() {s({center:{x:30,y:30},bottom:60,right:60}, rect1);});
	});
});

mocha.run();

setTimeout(window.close,1000);
